/*
  vc +kick13m -c99 -sc -sd -O2 -o send7 send7.c -lamigas
  send7 path:to/file
*/

#include <proto/exec.h>
#include <proto/dos.h>
#include <clib/alib_protos.h>
#include <dos/dos.h>
#include <devices/serial.h>

struct DosLibrary *DOSBase;

int strlen(char *s)
{
  int n = 0;
  while (*s++)
  {
    n++;
  }
  return n;
}

int main(char *filename)
{
  int retval = 20;
  if ((DOSBase = (struct DosLibrary *) OpenLibrary("dos.library", 0)))
  {
    BPTR lock;
    if ((lock = Lock(filename, ACCESS_READ)))
    {
      struct FileInfoBlock fib;
      if (Examine(lock, &fib))
      {
        if (fib.fib_DirEntryType < 0) /* is a file */
        {
          LONG inbytes = fib.fib_Size;
          LONG msgbytes = (((inbytes << 3) + 6) / 7) + 3;
          BPTR file;
          if ((file = Open(filename, MODE_OLDFILE)))
          {
            UBYTE *buffer;
            if ((buffer = AllocMem(msgbytes, 0L)))
            {
              /*
                  read file into end part of larger message buffer
                  ensyx and encode7 into earlier part of buffer
              */
              UBYTE *msg = buffer;
              UBYTE *in = msg + (msgbytes - inbytes);
              if (inbytes == Read(file, in, inbytes))
              {
                *msg++ = 0xF0;
                *msg++ = 0x7D;
                UBYTE output = 0;
                UBYTE shift = 1;
                for (LONG i = 0; i < inbytes; ++i)
                {
                  UBYTE input = *in++;
                  output |= (input >> shift);
                  *msg++ = output;
                  output = (input & ((1 << shift) - 1)) << (7 - shift);
                  if (++shift == 8)
                  {
                    *msg++ = output;
                    shift = 1;
                    output = 0;
                  }
                }
                if (shift != 1)
                {
                  *msg++ = output;
                }
              }
              *msg++ = 0xF7;
              /*
                  send buffer via MIDI
              */
              struct MsgPort *port;
              if ((port = CreatePort(NULL, 0)))
              {
                struct IOExtSer *io;
                if ((io = (struct IOExtSer *) CreateExtIO(port, sizeof(struct IOExtSer))))
                {
                  if (0 == OpenDevice("serial.device", 0, (struct IORequest *) io, 0L))
                  {
                    io->IOSer.io_Command = SDCMD_SETPARAMS;
                    io->io_Baud = 31250;
                    io->io_ReadLen = 8;
                    io->io_StopBits = 1;
                    io->io_SerFlags = SERF_XDISABLED | SERF_RAD_BOOGIE;
                    if (0 == DoIO((struct IORequest *) io))
                    {
                      BPTR out = Output();
                      Write(out, "Sending '", 9);
                      Write(out, filename, strlen(filename));
                      Write(out, "'... ", 5);
                      io->IOSer.io_Command = CMD_WRITE;
                      io->IOSer.io_Length = msgbytes;
                      io->IOSer.io_Data = buffer;
                      if (0 == DoIO((struct IORequest *) io))
                      {
                        Write(out, "OK!\n", 4);
                        retval = 0;
                      }
                      else
                      {
                        Write(out, "FAILED!\n", 8);
                      }
                    }
                    CloseDevice((struct IORequest *) io);
                  }
                  DeleteExtIO((struct IORequest *) io);
                }
                DeletePort(port);
              }
              FreeMem(buffer, msgbytes);
            }
            Close(file);
          }
        }
      }
      UnLock(lock);
    }
    CloseLibrary((struct Library *) DOSBase);
  }
  return retval;
}
